import pytest

from petisco.application.config.config import Config


@pytest.mark.end2end
def test_should_load_complete_petisco_yml(petisco_yml_path_end2end):

    filename = f"{petisco_yml_path_end2end}/petisco.all.yml"

    config = Config.from_filename(filename).unwrap()

    assert len(config.config_tasks.tasks.keys()) == 3



import pytest

from petisco.domain.value_objects.correlation_id import CorrelationId
from petisco.domain.value_objects.user_id import UserId
from petisco.frameworks.flask.flask_extension_is_installed import (
    flask_extension_is_installed,
)


@pytest.mark.end2end
@pytest.mark.skipif(
    not flask_extension_is_installed(), reason="Flask extension is not installed"
)
def test_should_return_200_when_call_healthcheck_with_happy_path(client_app):
    headers = {"Accept": "application/json"}
    response = client_app.get("/petisco/healthcheck", headers=headers)

    assert response.status_code == 200
    assert "app_name" in response.json
    assert "app_version" in response.json
    assert (
        "persistence_available" in response.json
        and response.json["persistence_available"] is True
    )


@pytest.mark.end2end
@pytest.mark.skipif(
    not flask_extension_is_installed(), reason="Flask extension is not installed"
)
def test_should_return_200_when_call_environment_with_happy_path_with_apikey(
    client_app, given_any_apikey
):
    headers = {"Accept": "application/json", "apikey": given_any_apikey}
    response = client_app.get("/petisco/environment", headers=headers)
    assert response.status_code == 200


@pytest.mark.end2end
@pytest.mark.skipif(
    not flask_extension_is_installed(), reason="Flask extension is not installed"
)
def test_should_return_401_when_call_environment_with_happy_path_without_apikey(
    client_app,
):
    headers = {"Accept": "application/json"}
    response = client_app.get("/petisco/environment", headers=headers)
    assert response.status_code == 401


@pytest.mark.end2end
@pytest.mark.skipif(
    not flask_extension_is_installed(), reason="Flask extension is not installed"
)
def test_should_return_200_when_call_sum_with_valid_values(client_app):
    headers = {
        "Content-Type": "multipart/form-data",
        "User-Agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36",
    }

    multipart_data = dict(value_1=2, value_2=3)
    response = client_app.post(
        "/petisco/sum",
        headers=headers,
        data=multipart_data,
        content_type="multipart/form-data",
    )
    assert response.status_code == 200
    assert response.json["result"] == 5


@pytest.mark.end2end
@pytest.mark.skipif(
    not flask_extension_is_installed(), reason="Flask extension is not installed"
)
def test_should_return_200_when_call_sum_with_valid_values_with_external_headers(
    client_app,
):
    headers = {
        "Content-Type": "multipart/form-data",
        "User-Agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36",
        "X-Correlation-Id": CorrelationId.generate().value,
        "X-Onboarding-Clientid": "petisco",
        "X-Onboarding-Userid": UserId.generate().value,
    }

    multipart_data = dict(value_1=2, value_2=3)
    response = client_app.post(
        "/petisco/sum",
        headers=headers,
        data=multipart_data,
        content_type="multipart/form-data",
    )
    assert response.status_code == 200
    assert response.json["result"] == 5


@pytest.mark.end2end
@pytest.mark.skipif(
    not flask_extension_is_installed(), reason="Flask extension is not installed"
)
def test_should_return_400_when_call_sum_without_required_value(client_app):
    headers = {"Content-Type": "multipart/form-data"}
    multipart_data = dict(value_1=2)
    response = client_app.post(
        "/petisco/sum",
        headers=headers,
        data=multipart_data,
        content_type="multipart/form-data",
    )
    assert response.status_code == 400



import pytest

from petisco.event.legacy.routing_key import RoutingKey


@pytest.mark.unit
def test_should_create_an_routing_key_and_match_fullname():

    routing_key = RoutingKey("acme.service.1.event.user.created")

    assert routing_key.match_fullname("acme.service.1.event.user.created")


@pytest.mark.unit
@pytest.mark.parametrize(
    "organization,service,version,type_routing_key,expected_result",
    [
        ("acme", "service", 1, "event", True),
        ("acme", None, None, None, True),
        (None, "service", None, None, True),
        (None, None, 1, None, True),
        (None, None, None, "event", True),
        ("invalid", "service", 1, "event", False),
        ("acme", "invalid", 1, "event", False),
        ("invalid", "service", -1, "event", False),
        ("invalid", "service", 1, "invalid", False),
    ],
)
def test_should_create_an_routing_key_and_match(
    organization, service, version, type_routing_key, expected_result
):

    routing_key = RoutingKey("acme.service.1.event.user.created")

    assert (
        routing_key.match(
            organization=organization,
            service=service,
            version=version,
            type_routing_key=type_routing_key,
        )
        == expected_result
    )



from typing import List

from meiga import Result, Error, Success, Failure, isSuccess, BoolResult
from meiga.decorators import meiga

from petisco.domain.aggregate_roots.info_id import InfoId
from petisco.security.token_decoder.interface_token_decoder import ITokenDecoder
from petisco.security.token_decoder.invalid_token_error import InvalidTokenError
from petisco.security.token_decoder.token import Token
from petisco.security.token_decoder.token_decoder import TokenDecoder
from petisco.security.token_manager.accepted_token import AcceptedToken
from petisco.security.token_manager.interface_token_manager import ITokenManager


class TokenManager(ITokenManager):
    def __init__(
        self,
        accepted_tokens: List[AcceptedToken],
        token_decoder: ITokenDecoder = TokenDecoder(),
    ):
        self.accepted_tokens = accepted_tokens
        self.token_decoder = token_decoder

    @meiga
    def execute(self, headers: dict) -> Result[InfoId, Error]:
        auth_token = headers.get("Authorization")

        if not auth_token:
            return Failure(
                InvalidTokenError(
                    message=f"This entry point expects a valid token: {[token.token_type for token in self.accepted_tokens]}"
                )
            )

        token = self.token_decoder.execute(auth_token).unwrap_or_return()

        self.ensure_required_token(token).unwrap_or_return()

        info_id = InfoId.from_token(token).update_from_headers(headers)

        return Success(info_id)

    @meiga
    def ensure_required_token(self, token: Token) -> BoolResult:
        result = isSuccess
        for accepted_token in self.accepted_tokens:
            result = accepted_token.check(token)
            if result.is_success:
                break
        result.unwrap_or_return()
        return isSuccess



from typing import Dict

from petisco.event.bus.domain.interface_event_bus import IEventBus
from petisco.event.shared.domain.event import Event


class NotImplementedEventBus(IEventBus):
    def info(self) -> Dict:
        return {"name": self.__class__.__name__}

    def publish(self, event: Event):
        if hasattr(self, "info_id"):
            event = event.add_info_id(self.info_id)

        if hasattr(self, "additional_meta"):
            event = event.update_meta(self.additional_meta)

        if not event or not issubclass(event.__class__, Event):
            raise TypeError("Bus only publishes petisco.Event objects")

    def retry_publish_only_on_store_queue(self, event: Event):
        if not event or not issubclass(event.__class__, Event):
            raise TypeError("Bus only publishes petisco.Event objects")

    def stop(self):
        pass



from datetime import datetime, timedelta

TIME_FORMAT = "%Y-%m-%d %H:%M:%S"


class DeployChecker:
    def __init__(self, deploy_time: str, courtesy_minutes: int):
        self.deploy_time = deploy_time
        deploy_datetime = datetime.strptime(self.deploy_time[:-6], TIME_FORMAT)
        self.comparison_time = deploy_datetime + timedelta(minutes=courtesy_minutes)

    def was_recently_deployed(self, now: datetime) -> bool:
        return now < self.comparison_time

    def get_deploy_time(self) -> str:
        return self.deploy_time



import pytest
from meiga import Success
from meiga.assertions import assert_failure
from meiga.decorators import meiga

from petisco.domain.value_objects.client_id import ClientId
from petisco.domain.errors.given_input_is_not_valid_error import (
    GivenInputIsNotValidError,
)
from petisco.domain.errors.length_limit_string_value_object_error import (
    ExceedLengthLimitValueObjectError,
)


@pytest.mark.unit
def test_should_declare_a_valid_client_id():
    value = "Acme"
    client_id = ClientId(value)

    assert isinstance(client_id, ClientId)
    assert client_id.value == value


@pytest.mark.unit
def test_should_declare_a_client_id_with_empty_string():
    value = ""
    client_id = ClientId(value)

    assert isinstance(client_id, ClientId)
    assert client_id.value == value


@pytest.mark.unit
def test_should_declare_a_name_that_exceeds_default_length_limits():

    with pytest.raises(ExceedLengthLimitValueObjectError):
        ClientId(
            "La Corporación Acme es una empresa ficticia, que existe en el universo de los Looney Tunes. Apareció la mayor cantidad de veces en las caricaturas de El Coyote y el Correcaminos, que hicieron famosa a Acme por sus productos peligrosos y poco reales, los cuales fallaban catastróficamente de las peores maneras."
        )


@pytest.mark.unit
def test_should_declare_a_name_with_js_injection():
    with pytest.raises(GivenInputIsNotValidError):
        ClientId("<script>evil()</script>")


@pytest.mark.unit
def test_should_fail_when_declare_a_non_valid_client_id_call_guard():
    @meiga
    def controller():
        user_id = ClientId("<script>evil()</script>")
        return Success(user_id)

    result = controller()
    assert_failure(result, value_is_instance_of=GivenInputIsNotValidError)



# import os
#
# from petisco import Petisco
#
# ROOT_PATH = os.path.dirname(os.path.abspath(__file__))
#
#
# def petisco_config():
#     petisco = Petisco.from_filename(ROOT_PATH + "/petisco.yml")
#     petisco.configure_events(ROOT_PATH + "/petisco.events.yml")
#
#
# def main():
#     petisco_config()
#     Petisco.get_instance().start()
#
#
# if __name__ == "__main__":
#     main()



from typing import Dict

from meiga import Result, Error, isSuccess, Success, Failure

from petisco.domain.value_objects.client_id import ClientId
from petisco.domain.value_objects.user_id import UserId
from tests.end2end.flask_app.shared.domain.repositories.interface_user_repository import (
    IUserRepository,
)
from tests.end2end.flask_app.shared.domain.repositories.user_not_found_error import (
    UserNotFoundError,
)
from tests.end2end.flask_app.user.domain.aggregate_roots.user import User


class InMemoryUserRepository(IUserRepository):
    def __init__(self):
        self.users = {}

    def info(self) -> Dict:
        return {"name": self.__class__.__name__}

    def save(self, user: User) -> Result[bool, Error]:
        self.users[f"{user.client_id}_{user.user_id}"] = user
        return isSuccess

    def retrieve(self, client_id: ClientId, user_id: UserId) -> Result[User, Error]:
        if f"{client_id}_{user_id}" in self.users:
            return Success(self.users[f"{client_id}_{user_id}"])
        else:
            return Failure(UserNotFoundError())



import os

import pytest
from petisco import NotifierMessage
from petisco.notifier.domain.notifier_exception_message import NotifierExceptionMessage
from petisco.notifier.infrastructure.slack.slack_notifier import SlackNotifier
from tests.modules.shared.info_id_mother import InfoIdMother

LONG_TEXT = "long_text" * 1000


@pytest.mark.skipif(
    condition=os.getenv("SLACK_TOKEN") is None,
    reason="Slack token is a github secret only available in CI workflows",
)
@pytest.mark.integration
class TestSlackNotifier:
    @staticmethod
    def get_slack_notifier():
        slack_token = os.getenv("SLACK_TOKEN")
        slack_tests_channel = os.getenv("SLACK_TESTS_CHANNEL")
        return SlackNotifier(token=slack_token, channel=slack_tests_channel)

    def test_publish_should_send_notification(self):
        notifier = self.get_slack_notifier()
        notifier_message = NotifierMessage(title="Test title", message="test message")
        notifier.publish(notifier_message=notifier_message)

    def test_publish_exception_should_send_notification(self):
        import traceback

        notifier = self.get_slack_notifier()
        try:
            raise Exception(LONG_TEXT)
        except Exception as exception:
            notifier_exception_message = NotifierExceptionMessage(
                exception=exception,
                input_parameters={"key": LONG_TEXT},
                executor="executor",
                traceback=traceback.format_exc() + LONG_TEXT,
                info_id=InfoIdMother.random(),
                info_petisco={},
            )
            notifier.publish(notifier_message=notifier_exception_message)



from pika import BasicProperties
from pika.exceptions import ChannelClosedByBroker

from petisco.event.bus.domain.interface_event_bus import IEventBus
from petisco.event.configurer.infrastructure.rabbitmq_event_configurer import (
    RabbitMqEventConfigurer,
)
from petisco.event.shared.infrastructure.rabbitmq.rabbitmq_consumer_connector import (
    RabbitMqConsumerConnector,
)
from petisco.event.shared.infrastructure.rabbitmq.rabbitmq_event_queue_name_formatter import (
    RabbitMqEventQueueNameFormatter,
)
from petisco.event.shared.domain.event import Event


class RabbitMqConsumerEventBus(IEventBus):
    def __init__(
        self, connector: RabbitMqConsumerConnector, organization: str, service: str
    ):
        self.connector = connector
        self.exchange_name = f"{organization}.{service}"
        self.rabbitmq_key = f"publisher-{self.exchange_name}"
        self.configurer = RabbitMqEventConfigurer(connector, organization, service)
        self.properties = BasicProperties(delivery_mode=2)  # PERSISTENT_TEXT_PLAIN

    def publish(self, event: Event):
        if hasattr(self, "info_id"):
            event = event.add_info_id(self.info_id)

        if not event or not issubclass(event.__class__, Event):
            raise TypeError("Bus only publishes petisco.Event objects")

        try:
            channel = self.connector.get_channel(self.rabbitmq_key)
            routing_key = RabbitMqEventQueueNameFormatter.format(
                event, exchange_name=self.exchange_name
            )

            channel.basic_publish(
                exchange=self.exchange_name,
                routing_key=routing_key,
                body=event.to_json(),
                properties=self.properties,
            )
        except ChannelClosedByBroker:
            # If Event queue is not configured, it will be configured and publication retried.
            self.configurer.configure_event(event)
            self.publish(event)

    def retry_publish_only_on_store_queue(self, event: Event):
        if not event or not issubclass(event.__class__, Event):
            raise TypeError("Bus only publishes petisco.Event objects")
        channel = self.connector.get_channel(self.rabbitmq_key)
        channel.basic_publish(
            exchange=self.exchange_name,
            routing_key="retry.store",
            body=event.to_json(),
            properties=self.properties,
        )

    def close(self):
        self.connector.close(self.rabbitmq_key)



from petisco import Webhook, Secret

DEFAULT_POST_URL = "http://127.0.0.1:5000/webhook"
DEFAULT_API_KEY = "'b0b905d6-228f-44bf-a130-c85d7aecd765"
DEFAULT_SECRET = Secret("9464a12f2186cdfe38fd1cfd797758c47c544e99")
DEFAULT_EVENT_NAME = "user_created"
DEFAULT_EVENT_VERSION = "1"


class WebhookMother:
    @staticmethod
    def default():
        return Webhook.create(
            post_url=DEFAULT_POST_URL,
            active=True,
            api_key=DEFAULT_API_KEY,
            secret=DEFAULT_SECRET,
            event_name=DEFAULT_EVENT_NAME,
            event_version=DEFAULT_EVENT_VERSION,
        )

    @staticmethod
    def not_active():
        return Webhook.create(
            post_url=DEFAULT_POST_URL,
            active=False,
            api_key=DEFAULT_API_KEY,
            secret=DEFAULT_SECRET,
            event_name=DEFAULT_EVENT_NAME,
            event_version=DEFAULT_EVENT_VERSION,
        )

    @staticmethod
    def with_invalid_url():
        return Webhook.create(
            post_url="invalid_url",
            active=True,
            api_key=DEFAULT_API_KEY,
            secret=DEFAULT_SECRET,
            event_name=DEFAULT_EVENT_NAME,
            event_version=DEFAULT_EVENT_VERSION,
        )



import connexion
import yaml
from flask_bcrypt import Bcrypt
from flask_cors import CORS

from petisco.frameworks.flask.application.json_encoder import JSONEncoder
from petisco.frameworks.interface_application import IApplication


class FlaskApplication(IApplication):
    def __init__(
        self,
        application_name: str,
        swagger_dir: str,
        config_file: str,
        port: int = 8080,
    ):
        self.application_name = application_name
        self.app = connexion.App(__name__, specification_dir=swagger_dir)
        self.config_file = config_file
        self.port = port
        self._configure()

    def _configure(self):
        CORS(self.app.app)
        self.app.app.config["BCRYPT_HANDLE_LONG_PASSWORDS"] = True
        self.bcrypt = Bcrypt(self.app.app)
        self.app.app.json_encoder = JSONEncoder
        self.app.add_api(
            self.config_file,
            arguments={"title": self.application_name},
            options={"serve_spec": False},
        )
        self.app.app.add_url_rule(
            rule=f"/{self.application_name}/openapi.json",
            view_func=self._openapi_json,
            endpoint=f"/{self.application_name}./{self.application_name}_openapi_json",
        )

    def start(self):
        self.app.run(port=self.port)

    def get_config(self):
        return self.config

    def get_app(self):
        return self.app

    def _openapi_json(self):
        openapi_yaml_path = f"{self.app.specification_dir}/{self.config_file}"

        with open(openapi_yaml_path) as openapi_yaml:
            openapi = yaml.safe_load(openapi_yaml)

        hidden_endpoints = []
        hidden_request_bodies = []
        for path, methods in openapi["paths"].items():
            for method, endpoint in methods.items():
                if endpoint.get("x-hidden"):
                    hidden_endpoints.append((path, method))
                if openapi["paths"][path][method].get("requestBody"):
                    if openapi["paths"][path][method]["requestBody"].get("content"):
                        for content_type, schema in openapi["paths"][path][method]["requestBody"]["content"].items():
                            if openapi["paths"][path][method]["requestBody"]["content"][content_type]["schema"]:
                                if openapi["paths"][path][method]["requestBody"]["content"][content_type]["schema"].get(
                                        "x-hidden"):
                                    hidden_request_bodies.append((path, method, content_type, len(
                                        openapi["paths"][path][method]["requestBody"]["content"].items()) == 1))

        for path, method in hidden_endpoints:
            del openapi["paths"][path][method]

        for path, method, content_type, only_one in hidden_request_bodies:
            if only_one:
                del openapi["paths"][path][method]["requestBody"]
            else:
                del openapi["paths"][path][method]["requestBody"]["content"][content_type]

        return openapi

    config = property(get_config)



import logging


def logging_config_func():
    logging.getLogger("PIL").setLevel(logging.WARNING)
    logging.getLogger("openapi_spec_validator").setLevel(logging.WARNING)
    logging.getLogger("connexion").setLevel(logging.WARNING)
    logging.getLogger("pika").setLevel(logging.WARNING)



from typing import Any

from meiga import Error

from petisco.domain.base_object import BaseObject
from dataclasses_json import config
from dataclasses import field, MISSING

ValueObject = BaseObject

ValueObjectError = Error


def value_object_field(value_object_class=ValueObject, default: Any = MISSING):
    def _field():
        return field(
            default=default,
            metadata=config(
                encoder=lambda result: result.value if result else None,
                decoder=lambda value: value_object_class(value) if value else None,
            ),
        )

    return _field()



import pytest

from petisco import RabbitMqConnector

from petisco.fixtures.testing_decorators import testing_with_rabbitmq
from tests.modules.event.mothers.defaults import DEFAULT_EXCHANGE_NAME
from tests.modules.event.mothers.rabbitmq_event_bus_mother import RabbitMqEventBusMother
from tests.modules.event.mothers.rabbitmq_event_configurer_mother import (
    RabbitMqEventConfigurerMother,
)


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_recover_from_connection_error_when_publish_an_event(
    make_user_created_event,
):
    connector = RabbitMqConnector()
    original_wait_seconds_retry = connector.wait_seconds_retry
    connector.wait_seconds_retry = 0.1

    configurer = RabbitMqEventConfigurerMother.default(connector)

    event = make_user_created_event()

    configurer.configure_event(event)

    bus = RabbitMqEventBusMother.default(connector)

    connection = connector.get_connection(DEFAULT_EXCHANGE_NAME)

    connection.close()

    bus.publish(event)

    connector.wait_seconds_retry = original_wait_seconds_retry

    configurer.clear()



from time import sleep

import pytest
from meiga import isSuccess, BoolResult

from petisco import Event, IEventBus, event_handler
from petisco.event.shared.domain.event_subscriber import EventSubscriber

from petisco.fixtures.testing_decorators import testing_with_rabbitmq
from tests.modules.event.mothers.event_user_created_mother import EventUserCreatedMother
from tests.modules.event.mothers.event_user_updated_mother import EventUserUpdatedMother
from tests.modules.event.mothers.rabbitmq_event_bus_mother import RabbitMqEventBusMother
from tests.modules.event.mothers.rabbitmq_event_configurer_mother import (
    RabbitMqEventConfigurerMother,
)
from tests.modules.event.mothers.rabbitmq_event_consumer_mother import (
    RabbitMqEventConsumerMother,
)
from tests.modules.event.spies.spy_events import SpyEvents
from tests.modules.unit.mocks.fake_logger import FakeLogger


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_publish_event_on_consumer():
    spy = SpyEvents()

    def assert_consumer(event: Event, event_bus: IEventBus) -> BoolResult:
        spy.append(event)
        event = EventUserUpdatedMother.random()
        event_bus.publish(event)
        return isSuccess

    event = EventUserCreatedMother.random()
    subscribers = [
        EventSubscriber(
            event_name=event.event_name,
            event_version=event.event_version,
            handlers=[assert_consumer],
        )
    ]

    configurer = RabbitMqEventConfigurerMother.with_retry_ttl_10ms()
    configurer.configure_subscribers(subscribers)

    consumer = RabbitMqEventConsumerMother.with_max_retries(1)
    consumer.add_subscribers(subscribers)
    consumer.start()

    bus = RabbitMqEventBusMother.default()
    for _ in range(5):
        event = EventUserCreatedMother.random()
        bus.publish(event)

    sleep(1.0)

    consumer.stop()
    configurer.clear()

    spy.assert_number_unique_events(5)


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_publish_event_on_consumer_with_event_handler():
    spy = SpyEvents()

    @event_handler(logger=FakeLogger())
    def assert_consumer(event: Event, event_bus: IEventBus) -> BoolResult:
        spy.append(event)
        event = EventUserUpdatedMother.random()
        event_bus.publish(event)
        return isSuccess

    event = EventUserCreatedMother.random()
    subscribers = [
        EventSubscriber(
            event_name=event.event_name,
            event_version=event.event_version,
            handlers=[assert_consumer],
        )
    ]
    configurer = RabbitMqEventConfigurerMother.with_retry_ttl_10ms()
    configurer.configure_subscribers(subscribers)

    consumer = RabbitMqEventConsumerMother.with_max_retries(1)
    consumer.add_subscribers(subscribers)
    consumer.start()

    bus = RabbitMqEventBusMother.default()
    for _ in range(1):
        event = EventUserCreatedMother.random()
        bus.publish(event)

    sleep(1.0)

    consumer.stop()
    configurer.clear()

    spy.assert_number_unique_events(1)



from typing import List

from petisco.event.configurer.infrastructure.rabbitmq_event_store_configurer import (
    RabbitMqEventStoreConfigurer,
)
from petisco.event.configurer.infrastructure.rabbitmq_event_subscribers_configurer import (
    RabbitMqEventSubcribersConfigurer,
)
from petisco.event.queue.domain.queue_config import QueueConfig
from petisco.event.shared.domain.event import Event
from petisco.event.shared.domain.event_subscriber import EventSubscriber
from petisco.event.configurer.domain.interface_event_configurer import IEventConfigurer
from petisco.event.shared.infrastructure.rabbitmq.rabbitmq_connector import (
    RabbitMqConnector,
)


class RabbitMqEventConfigurer(IEventConfigurer):
    def __init__(
        self,
        connector: RabbitMqConnector,
        organization: str,
        service: str,
        queue_config: QueueConfig = QueueConfig.default(),
        use_store_queues: bool = True,
    ):
        self._use_store_queues = use_store_queues
        self.event_subscribers_configurer = RabbitMqEventSubcribersConfigurer(
            connector, organization, service, queue_config
        )
        self.event_store_configurer = RabbitMqEventStoreConfigurer(
            connector, organization, service, queue_config
        )

    def configure(self):
        self.configure_subscribers([])

    def configure_event(self, event: Event):
        self.configure_subscribers(
            [
                EventSubscriber(
                    event_name=event.event_name,
                    event_version=event.event_version,
                    handlers=[],
                )
            ]
        )

    def configure_subscribers(
        self,
        subscribers: List[EventSubscriber],
        clear_subscriber_before: bool = False,
        clear_store_before: bool = False,
    ):
        if subscribers is None:
            subscribers = []

        if clear_subscriber_before:
            self.event_subscribers_configurer.clear_subscribers(subscribers)
        if clear_store_before and self._use_store_queues:
            self.event_store_configurer.clear()

        self.event_subscribers_configurer.execute(subscribers)
        if self._use_store_queues:
            self.event_store_configurer.execute()

    def clear(self):
        self.event_subscribers_configurer.clear()
        if self._use_store_queues:
            self.event_store_configurer.clear()



from time import sleep

import pytest
from meiga import isSuccess, isFailure, BoolResult

from petisco import Event, IEventBus

from petisco.fixtures.testing_decorators import testing_with_rabbitmq
from tests.modules.event.mothers.event_user_created_mother import EventUserCreatedMother
from tests.modules.event.mothers.rabbitmq_event_bus_mother import RabbitMqEventBusMother
from tests.modules.event.mothers.rabbitmq_event_configurer_mother import (
    RabbitMqEventConfigurerMother,
)
from tests.modules.event.mothers.rabbitmq_event_consumer_mother import (
    RabbitMqEventConsumerMother,
)
from tests.modules.event.spies.spy_events import SpyEvents


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_configure_two_services_without_subscribers():
    spy = SpyEvents()

    def assert_store(event: Event) -> BoolResult:
        spy.append(event)
        return isSuccess

    configurer_service_1 = RabbitMqEventConfigurerMother.with_service("service1")
    configurer_service_1.configure()

    configurer_service_2 = RabbitMqEventConfigurerMother.with_service("service2")
    configurer_service_2.configure()

    event = EventUserCreatedMother.random()
    bus = RabbitMqEventBusMother.with_service("service1")
    bus.publish(event)

    consumer = RabbitMqEventConsumerMother.default()
    consumer.add_handler_on_store(assert_store)
    consumer.start()

    sleep(1.0)

    consumer.stop()
    configurer_service_1.clear()
    configurer_service_2.clear()

    spy.assert_number_unique_events(1)
    spy.assert_first_event(event)
    spy.assert_last_event(event)
    spy.assert_count_by_event_id(event.event_id, 1)


@pytest.mark.integration
@testing_with_rabbitmq
@pytest.mark.parametrize(
    "publish_events_service_1, publish_events_service_2, expected_unique_events, expected_total_received_events, simulated_results_store",
    [
        (
            1,
            0,
            1,
            6,
            [isFailure, isFailure, isFailure, isFailure, isFailure, isSuccess],
        ),
        (100, 0, 100, 100, 100 * [isSuccess]),
        (1, 0, 1, 2, [isFailure, isSuccess]),
        (1, 0, 1, 3, [isFailure, isFailure, isSuccess]),
        (1, 1, 2, 2, [isSuccess, isSuccess]),
        (5, 10, 15, 15, 15 * [isSuccess]),
        (5, 5, 10, 20, 50 * [isFailure, isSuccess]),
    ],
)
def test_should_configure_two_services_without_subscribers_and_consuming_event_from_store_queues(
    publish_events_service_1,
    publish_events_service_2,
    expected_unique_events,
    expected_total_received_events,
    simulated_results_store,
):
    spy = SpyEvents()

    def assert_store(event: Event) -> BoolResult:
        spy.append(event)
        result = simulated_results_store.pop(0)
        return result

    configurer_service_1 = RabbitMqEventConfigurerMother.with_service("service1")
    configurer_service_1.configure()

    configurer_service_2 = RabbitMqEventConfigurerMother.with_service("service2")
    configurer_service_2.configure()

    bus_service_1 = RabbitMqEventBusMother.with_service("service1")
    bus_service_2 = RabbitMqEventBusMother.with_service("service2")

    def publish_event(bus: IEventBus, times: int):
        for _ in range(times):
            event = EventUserCreatedMother.random()
            bus.publish(event)

    publish_event(bus_service_1, publish_events_service_1)
    publish_event(bus_service_2, publish_events_service_2)

    consumer = RabbitMqEventConsumerMother.with_service("service1")
    consumer.add_handler_on_store(assert_store)
    consumer.start()

    sleep(1.0)

    consumer.stop()
    configurer_service_1.clear()
    configurer_service_2.clear()

    spy.assert_number_unique_events(expected_unique_events)
    spy.assert_number_total_events(expected_total_received_events)



from typing import Dict, List, Optional

from petisco.domain.aggregate_roots.info_id import InfoId
from petisco.notifier.domain.notifier_exception_message import NotifierExceptionMessage
from petisco.notifier.domain.notifier_message import NotifierMessage
from petisco.notifier.infrastructure.slack.interface_slack_notifier_message_converter import (
    ISlackNotifierMessageConverter,
)


class SlackNotifierMessageConverter(ISlackNotifierMessageConverter):
    def __repr__(self):
        return super().__repr__()

    def __info_petisco_block(self, info_petisco: Dict) -> Dict:
        return {
            "type": "section",
            "text": {
                "type": "mrkdwn",
                "text": f":cookie: *Message from Petisco*\n*Application:* {info_petisco['app_name']} ({info_petisco['app_version']})\n*Petisco:* {info_petisco['petisco_version']}\n*Environment:* {info_petisco['environment']}",
            },
        }

    def _info_id_block(self, info_id: InfoId) -> Optional[Dict]:
        if not info_id:
            return None
        info_id_text = ":information_source: *InfoId*\n"
        info_id_available = False
        if info_id.client_id:
            info_id_text += f"*ClientId:* {info_id.client_id.value}\n"
            info_id_available = True
        if info_id.user_id:
            info_id_text += f"*UserId:* {info_id.user_id.value}\n"
            info_id_available = True
        if info_id.correlation_id:
            info_id_text += f"*CorrelationId:* {info_id.correlation_id.value}\n"
            info_id_available = True
        if info_id.correlation_id:
            info_id_text += f"*IP:* {info_id.ip}"
            info_id_available = True
        if not info_id_available:
            return None
        return {"type": "section", "text": {"type": "mrkdwn", "text": info_id_text}}

    def __get_common_blocks(self, notifier_message: NotifierMessage) -> List[Dict]:
        blocks = []
        if notifier_message.info_petisco:
            blocks.append(self.__info_petisco_block(notifier_message.info_petisco))
        if notifier_message.info_id:
            blocks.append(self._info_id_block(notifier_message.info_id))
        if len(blocks) > 0:
            blocks.append({"type": "divider"})
        return blocks

    def __get_blocks_exception_message(
        self, notifier_message: NotifierExceptionMessage
    ) -> List[Dict]:
        blocks = []

        exception_block = {
            "type": "section",
            "text": {
                "type": "mrkdwn",
                "text": f":fire: *Exception* \n*Class*: {notifier_message.exception.__class__} *Description:* {str(notifier_message.exception)[:2500]}",
            },
        }
        executor_block = {
            "type": "section",
            "text": {
                "type": "mrkdwn",
                "text": f":pushpin: *Executor*\n{notifier_message.executor}",
            },
        }

        input_parameters_block_text = ":arrow_right: *Input Parameters*\n"

        if notifier_message.input_parameters:
            for k, v in notifier_message.input_parameters.items():
                input_parameters_block_text += f"* {k}: {v}\n"
        else:
            input_parameters_block_text += "No data :heavy_multiplication_x:"

        input_parameters_block = {
            "type": "section",
            "text": {
                "type": "mrkdwn",
                "text": input_parameters_block_text[:2500],
            },
        }
        traceback_block = {
            "type": "section",
            "text": {
                "type": "mrkdwn",
                "text": f":scroll: *Traceback*\n```{notifier_message.traceback[:2500]}```",
            },
        }
        blocks += self.__get_common_blocks(notifier_message)
        blocks.append(exception_block)
        blocks.append(executor_block)
        blocks.append(input_parameters_block)
        blocks.append(traceback_block)
        return blocks

    def convert(self, notifier_message: NotifierMessage):
        if isinstance(notifier_message, NotifierExceptionMessage):
            blocks = self.__get_blocks_exception_message(notifier_message)
        else:
            blocks = []
            message_block = {
                "type": "section",
                "text": {
                    "type": "mrkdwn",
                    "text": f":envelope: *Message*\n{notifier_message.message}",
                },
            }
            blocks += self.__get_common_blocks(notifier_message)
            if notifier_message.title:
                title_block = {
                    "type": "section",
                    "text": {
                        "type": "mrkdwn",
                        "text": f":label: *Title*\n{notifier_message.title}",
                    },
                }
                blocks.append(title_block)
            blocks.append(message_block)
        return blocks



from datetime import timedelta


class TimeDeltaParser:
    @staticmethod
    def ms_from_timedelta(value: timedelta):
        if isinstance(value, timedelta):
            value = value.microseconds / 1000
        return value



from typing import Dict

from dataclasses import dataclass

from petisco.domain.aggregate_roots.info_id import InfoId
from petisco.notifier.domain.notifier_message import NotifierMessage


@dataclass
class NotifierExceptionMessage(NotifierMessage):
    def __init__(
        self,
        executor: str,
        exception: Exception,
        traceback: str,
        input_parameters: dict = None,
        info_id: InfoId = None,
        info_petisco: Dict = None,
    ):
        super().__init__(info_id=info_id, info_petisco=info_petisco)
        self.executor = executor
        self.exception = exception
        self.traceback = traceback
        self.input_parameters = input_parameters

    def __str__(self) -> str:
        return f"Error {self.executor}: {repr(self.exception.__class__)} {self.exception} | {self.traceback} | {self.input_parameters}"



from time import sleep

import pytest
from meiga import isSuccess, BoolResult, isFailure

from petisco import Event

from petisco.fixtures.testing_decorators import testing_with_rabbitmq
from tests.modules.event.mothers.event_user_created_mother import EventUserCreatedMother
from tests.modules.event.mothers.rabbitmq_event_bus_mother import RabbitMqEventBusMother
from tests.modules.event.mothers.rabbitmq_event_configurer_mother import (
    RabbitMqEventConfigurerMother,
)
from tests.modules.event.mothers.rabbitmq_event_consumer_mother import (
    RabbitMqEventConsumerMother,
)
from tests.modules.event.spies.spy_events import SpyEvents


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_publish_consume_from_store_queue_from_rabbitmq_with_stop_and_start():

    spy = SpyEvents()

    def assert_consumer_event_store(event: Event) -> BoolResult:
        spy.append(event)
        return isSuccess

    bus = RabbitMqEventBusMother.default()

    first_event = EventUserCreatedMother.random()
    configurer = RabbitMqEventConfigurerMother.default()
    configurer.configure_event(first_event)

    # First Start & Stop
    bus.publish(first_event)
    first_consumer = RabbitMqEventConsumerMother.default()
    first_consumer.add_handler_on_store(assert_consumer_event_store)
    first_consumer.start()
    sleep(1.0)
    first_consumer.stop()

    # Second Start & Stop
    second_event = EventUserCreatedMother.random()
    second_consumer = RabbitMqEventConsumerMother.default()
    second_consumer.add_handler_on_store(assert_consumer_event_store)
    second_consumer.start()
    bus.publish(second_event)
    sleep(1.0)
    second_consumer.stop()

    configurer.clear()

    spy.assert_number_unique_events(2)
    spy.assert_count_by_event_id(first_event.event_id, 1)
    spy.assert_count_by_event_id(second_event.event_id, 1)


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_unsubscribe_and_resume_handler_on_store_queue():

    spy = SpyEvents()
    spy_dead_letter = SpyEvents()

    def assert_consumer_event_store(event: Event) -> BoolResult:
        spy.append(event)
        return isSuccess

    def assert_consumer_event_dead_letter(event: Event) -> BoolResult:
        spy_dead_letter.append(event)
        return isSuccess

    bus = RabbitMqEventBusMother.default()

    first_event = EventUserCreatedMother.random()
    configurer = RabbitMqEventConfigurerMother.with_retry_ttl_10ms()
    configurer.configure_event(first_event)

    # Consumer configuration
    consumer = RabbitMqEventConsumerMother.default()
    consumer.add_handler_on_store(assert_consumer_event_store)
    consumer.add_handler_on_queue(
        "dead_letter.store", assert_consumer_event_dead_letter
    )
    consumer.start()
    bus.publish(first_event)
    sleep(1.0)
    consumer.unsubscribe_handler_on_queue("store")

    for i in range(10):
        bus.publish(EventUserCreatedMother.random())
    sleep(5.0)

    consumer.resume_handler_on_queue("store")
    second_event = EventUserCreatedMother.random()
    bus.publish(second_event)
    sleep(3.0)
    consumer.stop()
    configurer.clear()

    spy.assert_number_unique_events(2)
    spy.assert_count_by_event_id(first_event.event_id, 1)
    spy.assert_count_by_event_id(second_event.event_id, 1)

    spy_dead_letter.assert_number_unique_events(10)


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_unsubscribe_and_resume_handler_on_dead_letter_store_queue():

    spy_dead_letter = SpyEvents()

    def assert_consumer_event_store(event: Event) -> BoolResult:
        return isFailure

    def assert_consumer_event_dead_letter(event: Event) -> BoolResult:
        spy_dead_letter.append(event)
        return isSuccess

    bus = RabbitMqEventBusMother.default()

    first_event = EventUserCreatedMother.random()
    configurer = RabbitMqEventConfigurerMother.with_retry_ttl_10ms()
    configurer.configure_event(first_event)

    # Consumer configuration
    consumer = RabbitMqEventConsumerMother.default()
    consumer.add_handler_on_store(assert_consumer_event_store)
    consumer.add_handler_on_queue(
        "dead_letter.store", assert_consumer_event_dead_letter
    )
    consumer.start()

    for i in range(10):
        bus.publish(EventUserCreatedMother.random())
    sleep(5.0)

    consumer.unsubscribe_handler_on_queue("dead_letter.store")

    for i in range(10):
        bus.publish(EventUserCreatedMother.random())
    sleep(5.0)

    consumer.resume_handler_on_queue("dead_letter.store")

    for i in range(10):
        bus.publish(EventUserCreatedMother.random())
    sleep(5.0)

    consumer.stop()
    configurer.clear()

    spy_dead_letter.assert_number_unique_events(30)


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_raise_an_error_when_unsubscribe_an_nonexistent_queue():
    def assert_consumer_event_store(event: Event) -> BoolResult:
        return isSuccess

    event = EventUserCreatedMother.random()
    configurer = RabbitMqEventConfigurerMother.with_retry_ttl_10ms()
    configurer.configure_event(event)

    # Consumer configuration
    consumer = RabbitMqEventConsumerMother.default()
    consumer.add_handler_on_store(assert_consumer_event_store)
    consumer.start()

    with pytest.raises(IndexError) as excinfo:
        consumer.unsubscribe_handler_on_queue("nonexistent_queue")
        assert (
            "Cannot unsubscribe an nonexistent queue (nonexistent_queue). Please, check configured consumers"
            in str(excinfo.value)
        )

    consumer.stop()
    configurer.clear()


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_raise_an_error_when_resume_an_nonexistent_queue():
    def assert_consumer_event_store(event: Event) -> BoolResult:
        return isSuccess

    event = EventUserCreatedMother.random()
    configurer = RabbitMqEventConfigurerMother.with_retry_ttl_10ms()
    configurer.configure_event(event)

    # Consumer configuration
    consumer = RabbitMqEventConsumerMother.default()
    consumer.add_handler_on_store(assert_consumer_event_store)
    consumer.start()

    with pytest.raises(IndexError) as excinfo:
        consumer.resume_handler_on_queue("nonexistent_queue")
        assert (
            "Cannot resume an nonexistent queue (nonexistent_queue). Please, check configured consumers"
            in str(excinfo.value)
        )
    consumer.stop()
    configurer.clear()



from typing import Dict, List
from abc import ABCMeta, abstractmethod

from petisco.event.shared.domain.event import Event
from petisco.event.shared.domain.event_subscriber import EventSubscriber


class IEventConfigurer:

    __metaclass__ = ABCMeta

    def __repr__(self):
        return f"IEventConfigurer"

    @classmethod
    def info(cls) -> Dict:
        return {"name": cls.__name__}

    @abstractmethod
    def configure_subscribers(
        self,
        subscribers: List[EventSubscriber],
        clear_subscriber_before: bool = False,
        clear_store_before: bool = False,
    ):
        raise NotImplementedError

    @abstractmethod
    def configure_event(self, event: Event):
        raise NotImplementedError

    @abstractmethod
    def clear(self):
        raise NotImplementedError



from abc import abstractmethod, ABC
from typing import List, Dict


class IDatabase(ABC):
    def __init__(self, name: str, models: Dict = None):
        self.name = name
        self.models = models if models else {}

    def info(self) -> Dict:
        _info = {"name": self.name}
        if self.models:
            _info["models"] = self.models
        return _info

    @abstractmethod
    def create(self):
        raise NotImplementedError

    @abstractmethod
    def delete(self):
        raise NotImplementedError

    @abstractmethod
    def clear_data(self):
        raise NotImplementedError

    @abstractmethod
    def get_model(self, model_name: str):
        raise NotImplementedError

    @abstractmethod
    def get_model_names(self) -> List[str]:
        raise NotImplementedError



from typing import Dict

from petisco.event.shared.domain.event import Event
from petisco.event.legacy.publisher.domain.interface_event_publisher import (
    IEventPublisher,
)


class FakeEventPublisher(IEventPublisher):
    def __init__(self):
        self.sent_events = []

    def info(self) -> Dict:
        pass

    def publish(self, event: Event):
        self.sent_events.append(event)

    def get_sent_events(self):
        return self.sent_events
